@@ -138,21 +138,8 @@ def iterative_step_minimize_step(step_predictor,
             mask = np.delete(mask, done)
             if len(mask) == 0:
                 return step, errs, [], done
-    if unitary:
-        step = step[rem,]
-        norms = np.linalg.norm(step, axis=1)
-        # norms = norms[rem,]
-        v = np.linalg.norm(guess[rem,], axis=-1)
-        r = np.sqrt(norms[rem,] ** 2 + v ** 2)
-        step = guess[rem,] * (1/r[:, np.newaxis] - 1) + step / r[:, np.newaxis]
-        if generate_rotation:
-            raise NotImplementedError(">3D rotations are complicated")
-            axis = vec_ops.vec_crosses(g, guess[mask,])[0]
-            ang = np.arctan2(norms / r, v / r)
-            rot = tfs.rotation_matrix(axis, ang)
-            rotations = rot @ rotations
-    else:
-        step = step[rem,]
+
+    step = step[rem,]
 
     if prev_steps is not None:
         prev_steps = prev_steps[mask,]
@@ -162,7 +149,10 @@ def iterative_step_minimize_step(step_predictor,
         overlaps = np.reshape(overlaps, prev_steps.shape[:2])
         osc = np.where(
             np.all(
-                np.logical_and(oscillation_overlap_cutoff_min < abs(overlaps), abs(overlaps) < oscillation_overlap_cutoff_max),
+                np.logical_and(
+                    oscillation_overlap_cutoff_min < abs(overlaps),
+                    abs(overlaps) < oscillation_overlap_cutoff_max
+                ),
                 axis=1
             )
         )
@@ -174,12 +164,23 @@ def iterative_step_minimize_step(step_predictor,
                 osc = osc[0][osc2]
                 scaling = np.min([cur_norms[osc2], -dist[osc2] / 2], axis=0) / cur_norms[osc2]
                 step[osc,] = step[osc,] * scaling
-            if unitary:
-                norms = np.linalg.norm(step[osc,], axis=1)
-                # norms = norms[rem,]
-                v = np.linalg.norm(guess[rem,], axis=-1)
-                r = np.sqrt(norms ** 2 + v ** 2)
-                step = guess[rem,] * (1 / r[:, np.newaxis] - 1) + step / r[:, np.newaxis]
+
+    if unitary:
+        step = step
+        norms = np.linalg.norm(step, axis=1)
+        # norms = norms[rem,]
+        v = np.linalg.norm(guess[rem,], axis=-1)
+        r = np.sqrt(norms ** 2 + v ** 2)
+        step = guess[rem,] * (1/r[:, np.newaxis] - 1) + step / r[:, np.newaxis]
+        if generate_rotation:
+            raise NotImplementedError(">3D rotations are complicated")
+            axis = vec_ops.vec_crosses(g, guess[mask,])[0]
+            ang = np.arctan2(norms / r, v / r)
+            rot = tfs.rotation_matrix(axis, ang)
+            rotations = rot @ rotations
+    else:
+        step = step[rem,]
+
 
     return step, errs, mask, done
 
@@ -196,7 +197,7 @@ def iterative_step_minimize(
         orthogonal_directions=None,
         orthogonal_projection_generator=None,
         region_constraints=None,
-        convergence_metric=None,
+        function=None,
         max_displacement=None,
         max_displacement_norm=None,
         oscillation_damping_factor=None,
@@ -205,6 +206,7 @@ def iterative_step_minimize(
         tol=1e-8,
         use_max_for_error=True,
         max_iterations=100,
+        convergence_metric=None,
         track_best=False,
         logger=None
 ):
@@ -221,9 +223,14 @@ def iterative_step_minimize(
     if track_best:
         best = guess.copy()
         best_errs = np.full(guess.shape[0], -1, dtype=float)
+        if function is not None:
+            best_vals = np.full(guess.shape[0], np.inf, dtype=float)
+        else:
+            best_vals = None
     else:
         best = None
         best_errs = None
+        best_vals = None
     its = np.zeros(guess.shape[0], dtype=int)
     errs = np.zeros(guess.shape[0], dtype=float)
     mask = np.arange(guess.shape[0])
@@ -257,14 +264,26 @@ def iterative_step_minimize(
             )
             if best is not None:
                 if i == 0:
+                    if best_vals is not None:
+                        best_vals[mask,] = function(guess[mask,], mask)
                     best_errs[mask,] = step_errs
                     best[mask,] = guess[mask,]
                 else:
-                    improved = np.where(step_errs < best_errs[mask,])
-                    if len(improved[0]) > 0:
-                        imp_mask = mask[improved]
-                        best_errs[imp_mask,] = step_errs[improved,]
-                        best[imp_mask,] = guess[imp_mask,]
+                    if best_vals is not None:
+                        new_vals = function(guess[mask,], mask)
+                        improved = np.where(new_vals < best_vals[mask,])
+                        if len(improved[0]) > 0:
+                            imp_mask = mask[improved]
+                            best_vals[imp_mask,] = new_vals[improved,]
+                            best[imp_mask,] = guess[imp_mask,]
+                            best_errs[imp_mask,] = step_errs[improved,]
+                    else:
+                        improved = np.where(step_errs < best_errs[mask,])
+                        if len(improved[0]) > 0:
+                            imp_mask = mask[improved]
+                            best_errs[imp_mask,] = step_errs[improved,]
+                            best[imp_mask,] = guess[imp_mask,]
+
 
             logger.log_print("Predicted steps: {step}", step=step)
             logger.log_print("Step errors: {errs}", errs=step_errs)
@@ -273,7 +292,7 @@ def iterative_step_minimize(
                     prev_step = np.random.uniform(size=(step.shape[0], prevent_oscillations, step.shape[1])).astype(step.dtype)
                 p = i % prevent_oscillations
                 prev_step[new_mask, p] = step
-            if oscillation_damping_factor is not None:
+            if (not unitary) and oscillation_damping_factor is not None:
                 if prev_errs is None:
                     prev_errs = step_errs
                 else:
@@ -788,6 +807,7 @@ class _WolfeLineSearch(LineSearcher):
         return derphi
 
     def prep_search(self, initial_geom, search_dir):
+        raise NotImplementedError()
         a_guess, opts, phi = super().prep_search(initial_geom, search_dir)
         derphi = self._grad_func(self.grad, initial_geom, search_dir)
         gfk = self.grad(initial_geom)
@@ -955,12 +975,12 @@ class QuasiNewtonStepFinder:
         return self.hess_appx(guess, mask, projector=projector)
 
 class QuasiNetwonHessianApproximator:
-    orthogonal_dirs_cutoff = 1e-8
+    orthogonal_dirs_cutoff = 1e-16#1e-8
     line_search = ArmijoSearch
     def __init__(self, func, jacobian, initial_beta=1,
                  damping_parameter=None, damping_exponent=None,
                  line_search=True, restart_interval=10,
-                 restart_hessian_norm=1e-5,
+                 restart_hessian_norm=1e-12,
                  # approximation_mode='direct'
                  approximation_mode='inverse'
                  ):
@@ -1003,8 +1023,10 @@ class QuasiNetwonHessianApproximator:
         if cutoff is None:
             cutoff = cls.orthogonal_dirs_cutoff
         mask = np.full(norms[0].reshape(-1,).shape, True)
-        for n in norms:
-            mask = np.logical_and(mask, np.abs(n.reshape(-1)) > cutoff)
+        if cutoff is not None:
+            for n in norms:
+                mask = np.logical_and(mask, np.abs(n.reshape(-1)) > cutoff)
+
         # good_pos = np.where(np.logical_and(*[
         #     np.abs(n.reshape(-1, )) > cutoff
         #     for n in norms
@@ -1076,7 +1098,7 @@ class QuasiNetwonHessianApproximator:
 
 class BFGSApproximator(QuasiNetwonHessianApproximator):
 
-    orthogonal_dirs_cutoff = 1e-8
+    orthogonal_dirs_cutoff = 1e-16 #1e-8
     def get_hessian_update(self, identities, jacobian_diffs, prev_steps, prev_hess):
         I = identities
         dx = prev_steps[:, :, np.newaxis]
@@ -1410,13 +1432,14 @@ class SchelgelApproximator(BofillApproximator):
 class ConjugateGradientStepFinder:
     supports_hessian = False
 
-    def __init__(self, func, jacobian, approximation_type='fletcher-reeves', logger=None, **generator_opts):
+    def __init__(self, func, jacobian, approximation_type='polak-ribiere', logger=None, **generator_opts):
         self.step_appx = self.beta_approximations[approximation_type.lower()](func, jacobian, **generator_opts)
         self.logger = logger
     @property
     def beta_approximations(self):
         return {
-            'fletcher-reeves':FletcherReevesApproximator
+            'fletcher-reeves':FletcherReevesApproximator,
+            'polak-ribiere':PolakRibiereApproximator
         }
 
     def __call__(self, guess, mask, projector=None):
@@ -1430,7 +1453,7 @@ class ConjugateGradientStepApproximator:
 
     def __init__(self, func, jacobian,
                  damping_parameter=None, damping_exponent=None,
-                 restart_interval=50, restart_parameter=.1,
+                 restart_interval=50, restart_parameter=0.9,
                  line_search=True):
         self.func = func
         self.jac = jacobian
@@ -1506,7 +1529,14 @@ class ConjugateGradientStepApproximator:
 class FletcherReevesApproximator(ConjugateGradientStepApproximator):
     def get_beta(self, new_jacs, prev_jac, prev_step_dir):
         return (
-                (new_jacs[:, np.newaxis, :] @new_jacs[:, :, np.newaxis]) /
+                (new_jacs[:, np.newaxis, :] @ new_jacs[:, :, np.newaxis]) /
+                (prev_jac[:, np.newaxis, :] @ prev_jac[:, :, np.newaxis])
+        ).reshape(len(new_jacs))
+
+class PolakRibiereApproximator(ConjugateGradientStepApproximator):
+    def get_beta(self, new_jacs, prev_jac, prev_step_dir):
+        return (
+                (new_jacs[:, np.newaxis, :] @ (new_jacs[:, :, np.newaxis] - prev_jac[:, :, np.newaxis])) /
                 (prev_jac[:, np.newaxis, :] @ prev_jac[:, :, np.newaxis])
         ).reshape(len(new_jacs))
 
@@ -1780,7 +1810,7 @@ class NudgedElasticBandStepFinder(ChainMinimizingStepFinder):
         if prev is None or next is None: return jac
         self._last_tangent = self.get_tangent(guess, mask, cur, prev, next)
 
-        return vec_ops.project_out(jac, self._last_tangent[:, :, np.newaxis], orthornomal=True)
+        return vec_ops.project_out(jac, self._last_tangent[:, :, np.newaxis], orthonormal=True)
 
     def image_pairwise_contribution(self, guess, mask, cur, prev, next, order=0):
         if order > 2: return 0
