@@ -12,6 +12,8 @@ __all__ = [
     'contained',
     'difference',
     'find',
+    "fast_first_nonzero",
+    "fast_first_zero",
     'argsort',
     'group_by',
     'grouping_info',
@@ -21,6 +23,7 @@ __all__ = [
     "permutation_indices",
     "vector_ix",
     "vector_take",
+    "vector_take_ix",
     "index_mask",
     "index_complement"
 ]
@@ -593,6 +596,8 @@ def from_iter_nd(iter, dtype, shape, like=None, **extra):
 def permutation_indices(n, r, dtype=int):
     if r == 0:
         return np.array([[]])
+    n = int(n)
+    r = int(r)
     return from_iter_nd(
         itertools.permutations(range(n), r),
         shape=(math.factorial(n)//math.factorial(n-r), r),
@@ -650,6 +655,33 @@ def index_complement(shape, inds):
         for k in range(-len(shape), 0)
     )
 
+def vector_take_ix(base_shape, inds, shared=None):
+    if shared is None:
+        shared = len(base_shape) - 1
+
+    if isinstance(inds, tuple):
+        if is_numeric(inds[0]): inds = tuple(np.asanyarray(ii) for ii in inds)
+    else:
+        inds = (np.asanyarray(inds),)
+
+    if shared is None: shared = 0
+    shared_shape = base_shape[:shared]
+    bcast_shape_ind = inds[-1].shape[shared:-1]
+    bcast_shape_arr = base_shape[shared:-len(inds)]
+
+    total_shape = shared_shape + bcast_shape_arr + bcast_shape_ind
+
+    inds_nob = tuple(
+        np.broadcast_to(
+            np.expand_dims(x, list(range(shared, shared + len(bcast_shape_arr)))),
+            total_shape + x.shape[-len(inds):]
+        )
+        for x in inds
+    )
+    inds = vector_ix(base_shape[-len(inds):], inds_nob)
+
+    return inds
+
 def vector_take(arr, inds, shared=None, return_spec=False):
     """
     A generalized array indexing that broadcasts properly across everything except for the specified "take" index
@@ -721,3 +753,23 @@ def take_where_groups(arr, where, presorted=True, return_rows=False):
     else:
         return vals
 
+def fast_first_nonzero(arr, axis=-1):
+    arr = np.asanyarray(arr)
+    if not np.issubdtype(arr.dtype, np.integer):
+        raise TypeError(f"array of dtype {arr.dtype} can't be found with fast_first_nonzero")
+    arr = np.moveaxis(arr, axis, -1)
+    v = np.argmax(arr.view(bool), axis=-1)
+    r = v // arr.itemsize
+    mask = vector_take(arr, r[..., np.newaxis], shared=r.ndim) == 0
+    r[mask[..., 0]] = -1
+    return r
+
+def fast_first_zero(arr, axis=-1):
+    arr = np.moveaxis(arr, axis, -1)
+    if not np.issubdtype(arr.dtype, np.integer):
+        raise TypeError(f"array of dtype {arr.dtype} can't be found with fast_first_nonzero")
+    v = np.argmin(np.logical_nor(arr.view(bool)), axis=-1) # might not work...
+    r = v // arr.itemsize
+    mask = vector_take(arr, r) == 0
+    r[mask] = -1
+    return r
\ No newline at end of file
