@@ -17,6 +17,8 @@ __all__ = [
     "vec_norms",
     "vec_tensordot",
     "vec_tensordiag",
+    "vec_block_diag",
+    "diag_indices",
     "block_array",
     "vec_tdot",
     "distance_matrix",
@@ -49,6 +51,7 @@ __all__ = [
     "fractional_power",
     "unitarize_transformation",
     "maximum_similarity_transformation",
+    "polar_decomposition",
     "matrix_transform_from_eigs",
     "symmetric_matrix_exp",
     "imaginary_symmetric_matrix_exp",
@@ -110,6 +113,15 @@ def vec_norms(vecs, axis=-1):
     #     raise NotImplementedError("Norm along not-the-last axis not there yet...")
     return np.linalg.norm(vecs, axis=axis)
 
+def points_from_distance_matrix(dist_mat, test_idx=0, zero_cutoff=1e-8):
+    dist_mat = np.asanyarray(dist_mat)
+    d2 = dist_mat ** 2
+    dd = (d2[..., test_idx, :, np.newaxis] + d2[..., test_idx, np.newaxis, :] - d2[..., :, :])/2
+    s, u = np.linalg.eigh(dd)
+    ndim = np.max(np.sum(s > zero_cutoff, axis=-1))
+    vecs = u[..., :, -ndim:] * np.sqrt(s[..., np.newaxis, -ndim:])
+    return vecs
+
 def distance_matrix(pts, axis=-1, axis2=None, return_triu=False, return_indices=False):
     pts = np.asanyarray(pts)
     if axis2 is None:
@@ -177,7 +189,7 @@ def vec_handle_zero_norms(vecs, norms, axis=-1, zero_thresh=None):
     vecs = vecs * (1 - zeros.astype(int))
     return vecs, zeros
 
-def vec_normalize(vecs, norms=None, axis=-1, zero_thresh=None):
+def vec_normalize(vecs, norms=None, axis=-1, zero_thresh=None, return_norms=False):
     """
 
     :param vecs:
@@ -190,16 +202,23 @@ def vec_normalize(vecs, norms=None, axis=-1, zero_thresh=None):
     if norms is None:
         norms = vec_norms(vecs, axis=axis)
     vecs, zeros = vec_handle_zero_norms(vecs, norms, axis=axis, zero_thresh=zero_thresh)
-    norms = np.expand_dims(norms, axis)
-    norms[zeros] = Options.zero_placeholder # since we already zeroed out the vector
+    e_norms = np.expand_dims(norms, axis)
+    e_norms[zeros] = Options.zero_placeholder # since we already zeroed out the vector
 
-    return vecs/norms
+    if return_norms:
+        return vecs/e_norms, norms
+    else:
+        return vecs/e_norms
 
-def vec_rescale(vecs, target_range=None, axis=-1):
+def vec_rescale(vecs, target_range=None, cur_range=None, axis=-1):
     vecs = np.asanyarray(vecs)
-    cur_mins = np.expand_dims(np.min(vecs, axis=axis), axis)
-    cur_max = np.expand_dims(np.max(vecs, axis=axis), axis)
-    cur_range = cur_max - cur_mins
+    if cur_range is None:
+        cur_mins = np.expand_dims(np.min(vecs, axis=axis), axis)
+        cur_max = np.expand_dims(np.max(vecs, axis=axis), axis)
+        cur_range = cur_max - cur_mins
+    else:
+        cur_mins = cur_range[0]
+        cur_range = cur_range[1] - cur_range[0]
     rescaled = (vecs - cur_mins) / cur_range
     if target_range is not None:
         if util.is_numeric(target_range):
@@ -222,15 +241,22 @@ def vec_crosses(vecs1, vecs2, normalize=False, zero_thresh=None, axis=-1):
     crosses = np.cross(vecs1, vecs2, axis=axis)
     if normalize:
         norms = vec_norms(crosses, axis=axis)
-
-        if isinstance(norms, np.ndarray):
-            zero_thresh = Options.norm_zero_threshold if zero_thresh is None else zero_thresh
-            bad_norms = np.where(np.abs(norms) <= zero_thresh)
+        zero_thresh = Options.norm_zero_threshold if zero_thresh is None else zero_thresh
+        smol = util.is_numeric(norms)
+        if smol:
+            bad_norms = norms < zero_thresh
+            if bad_norms:
+                norms = np.array(1.)
+        else:
+            bad_norms = np.where(norms <= zero_thresh)
             norms[bad_norms] = 1.
 
         crosses = crosses/norms[..., np.newaxis]
 
-        if isinstance(norms, np.ndarray):
+        if smol:
+            if bad_norms:
+                crosses *= 0.
+        else:
             crosses[bad_norms] *= 0.
 
     return crosses
@@ -326,18 +352,28 @@ def vec_angles(vectors1, vectors2, norms=None, up_vectors=None, zero_thresh=None
 
     norm_prod = norms1*norms2
     if check_zeros:
-        if isinstance(norm_prod, np.ndarray):
-            zero_thresh = Options.norm_zero_threshold if zero_thresh is None else zero_thresh
+        zero_thresh = Options.norm_zero_threshold if zero_thresh is None else zero_thresh
+        smol = util.is_numeric(norm_prod)
+        if smol:
+            bad_norms = norm_prod < zero_thresh
+            if bad_norms:
+                norm_prod = np.array(1.)
+        else:
             bad_norm_prods = np.where(np.abs(norm_prod) <= zero_thresh)
             norm_prod[bad_norm_prods] = 1.
+
     cos_comps = dots/norm_prod
     cross_norms = vec_norms(crosses, axis=axis)
     sin_comps = cross_norms/norm_prod
 
     angles = np.arctan2(sin_comps, cos_comps)
 
-    if check_zeros and isinstance(norm_prod, np.ndarray):
-        angles[bad_norm_prods] = 0.
+    if check_zeros:
+        if smol:
+            if bad_norms:
+                angles = np.array(0.)
+        else:
+            angles[bad_norm_prods] = 0.
 
     # return signed angles
     if up_vectors is not None:
@@ -537,9 +573,24 @@ def block_array(blocks, ndim=2, padding=0):
     for k in range(blocks.ndim-ndim-padding):
         blocks = np.concatenate(blocks, axis=-(k+1))
     return blocks
+def vec_block_diag(mats, kroneckerize=True):
+    mats = np.asanyarray(mats)
+    base_shape = mats.shape[:-3]
+    stack_shape = mats.shape[-3]
+    arr_shape = mats.shape[-2:]
+    arr = np.zeros(base_shape + (stack_shape, stack_shape) + arr_shape)
+    inds = diag_indices(base_shape, stack_shape, k=2)
+    arr[inds] = mats
+    if kroneckerize:
+        arr = np.moveaxis(arr, -2, -3).reshape(base_shape + (stack_shape*arr_shape[-1], stack_shape*arr_shape[-2]))
+    return arr
+
 
 def identity_tensors(base_shape, ndim):
     eye = np.eye(ndim)
+    if util.is_int(base_shape):
+        base_shape = [base_shape]
+    base_shape = tuple(base_shape)
     return np.broadcast_to(
         np.expand_dims(eye, list(range(len(base_shape)))),
         base_shape + (ndim, ndim)
@@ -1396,6 +1447,13 @@ def unitarize_transformation(tf):
     shared_dim = min((u.shape[-1], v.shape[-2]))
     return u[..., :, :shared_dim] @ v[..., :shared_dim, :]
 
+def polar_decomposition(tf):
+    u, s, v = np.linalg.svd(tf)
+    shared_dim = min((u.shape[-1], v.shape[-2]))
+    Q = u[..., :, :shared_dim] @ v[..., :shared_dim, :]
+    P = u @ vec_tensordiag(s) @ np.moveaxis(u, -1, -2)
+    return P, Q
+
 def maximum_similarity_transformation(basis, target, apply_transformation=True):
     lstsq_tf = np.linalg.lstsq(basis, target, rcond=None)[0]
     tf = unitarize_transformation(lstsq_tf)
