@@ -23,6 +23,7 @@ __all__ = [
     "Any",
     "Sign",
     "Number",
+    "IntBaseNumber",
     "Integer",
     "PositiveInteger",
     "ASCIILetter",
@@ -69,7 +70,8 @@ class RegexPattern:
                  handler=None, # a data handler for after capturing a block
                  default_value=None, # default value for Optional types
                  capturing=None,
-                 allow_inner_captures=False # whether or not to multiple captures for a given pattern
+                 allow_inner_captures=False, # whether or not to multiple captures for a given pattern
+                 escape=True
                  ):
         """
         :param pat:
@@ -110,6 +112,7 @@ class RegexPattern:
         self._pat = pat
         self._cached = None
         self._comp = None
+        self.escape = escape
 
         self.name = name # not used for anything but can make the code clearer
 
@@ -124,12 +127,17 @@ class RegexPattern:
         if children is None:
             children = []
         elif not isinstance(children, list):
-            if isinstance(children, str):
-                children = [ children ]
+            if isinstance(children, (str, RegexPattern)):
+                children = [children]
             else:
                 children = list(children)
 
-        children = [ c if isinstance(c, RegexPattern) else NonCapturing(c) for c in children ]
+        children = [
+            c
+                if isinstance(c, RegexPattern) else
+            NonCapturing(re.escape(c) if isinstance(c, str) else c)
+            for c in children
+        ]
         for c in children:
             c.add_parent(self)
         self._children = children
@@ -441,12 +449,12 @@ class RegexPattern:
             kids = [ group(k, no_capture=no_capture) if not is_grouped(k) else k for k in kids ]
         return joiner.join(kids)
     def build(self,
-              joiner = None,
-              prefix = None,
-              suffix = None,
-              recompile = True,
-              no_captures = False,
-              verbose = False
+              joiner=None,
+              prefix=None,
+              suffix=None,
+              recompile=True,
+              no_captures=False,
+              verbose=False
               ):
         # might want to add a flag that checks if a block has already been wrapped in no-capture? That would cut down
         # on regex length...
@@ -699,12 +707,22 @@ class RegexPattern:
     ### Supporting the re mechanisms directly for convenience
     def match(self, txt, *args):
         return re.match(self.compiled, txt, *args)
+    def fullmatch(self, txt, *args):
+        return re.fullmatch(self.compiled, txt, *args)
     def search(self, txt, *args):
         return re.search(self.compiled, txt, *args)
     def findall(self, txt, *args):
         return re.findall(self.compiled, txt, *args)
     def finditer(self, txt, *args):
         return re.finditer(self.compiled, txt, *args)
+    def sub(self, rep, txt, *args):
+        return re.sub(self.compiled, rep, txt, *args)
+    def subn(self, rep, txt, *args):
+        return re.subn(self.compiled, rep, txt, *args)
+    def replace(self, txt, replacement, *args):
+        return re.sub(self.compiled, replacement, txt, *args)
+    def remove(self, txt, *args):
+        return self.replace(txt, '', *args)
 
 ######################################################################################################################
 #
@@ -937,6 +955,13 @@ Number.__doc__ = """
     Represents a real number, like -1.23434; doesn't support "E" notation
     """
 
+num_p = opnb_p(sign_p)+r"\d+\.\d*" # real number
+IntBaseNumber = RegexPattern(num_p, "IntBaseNumber", dtype=float)
+IntBaseNumber.__name__ = "IntBaseNumber"
+IntBaseNumber.__doc__ = """
+    Represents a real number with definite integer part, like -1.23434 or 0.; doesn't support "E" notation
+    """
+
 int_p = opnb_p(sign_p)+r"\d+" # integer
 Integer = RegexPattern(int_p, "Integer", dtype=int)
 Integer.__name__ = "Integer"
