@@ -13,6 +13,7 @@ import enum, abc, contextlib, numpy as np
 import uuid
 
 from .. import Numputils as nput
+from .. import Devutils as dev
 
 from . import VTKInterface as vtk
 from ..ExternalPrograms import VPythonInterface as vpython
@@ -194,12 +195,16 @@ class GraphicsAxes(metaclass=abc.ABCMeta):
     @abc.abstractmethod
     def draw_line(self, points, **styles):
         ...
+    def draw_point(self, points, **styles):
+        return self.draw_disk(points, **styles)
     @abc.abstractmethod
     def draw_disk(self, points, **styles):
         ...
     @abc.abstractmethod
     def draw_rect(self, points, **styles):
         ...
+    def draw_triangle(self, points, **styles):
+        return self.draw_poly(points, **styles)
     @abc.abstractmethod
     def draw_poly(self, points, **styles):
         ...
@@ -935,7 +940,7 @@ class MPLAxes(GraphicsAxes):
 
     def draw_text(self, points, vals, **styles):
         points = np.asanyarray(points)
-        if points.ndim == 2:
+        if points.ndim == 1:
             points = points[np.newaxis]
         if isinstance(vals, str):
             vals = [vals]
@@ -1040,11 +1045,21 @@ class MPLFigure(GraphicsFigure):
         raise NotImplementedError(...)
     def close(self, *, backend):
         return backend.plt.close(self.obj)
+
+    _cb_opts = ("orientation", "extendfrac", "extendrect", "drawedges", "boundaries", "spacing")
     def create_colorbar(self, graphics, axes, norm=None, cmap=None, **kw):
         if graphics is None:
             import matplotlib.cm as cm
             graphics = cm.ScalarMappable(norm=norm, cmap=cmap)
-        self.obj.colorbar(graphics, cax=axes.obj, **kw)
+        cb_opts, fig_opts = dev.OptionsSet(kw).split(None, self._cb_opts)
+        self.obj.colorbar(graphics, cax=axes.obj, **cb_opts)
+        if len(fig_opts) > 0:
+            from .Graphics import Graphics
+            Graphics(
+                # parent=self,
+                figure=self,
+                axes=axes
+            ).set_options(**fig_opts)
         return axes
     def get_figure_label(self):
         return self.obj.suptitle()
@@ -1753,8 +1768,17 @@ class VPythonAxes(GraphicsAxes):
     def draw_line(self, points, **styles):
         return self.graph.plot(*np.asanyarray(points).T, **styles)
 
-    def draw_disk(self, points, color=None, **styles):
-        return self.graph.scatter(*np.asanyarray(points).T, **styles)
+    def draw_disk(self, points, color=None, edge_color=None, radius=1,
+                  edgecolors=None,
+                  s=None, **styles):
+        if edgecolors is None:
+            if edge_color is not None:
+                edgecolors = edge_color
+            else:
+                edgecolors=[[0.] * 3 + [.3]]
+        if s is None:
+            s = (10 * radius) ** 2
+        return self.graph.scatter(*np.asanyarray(points).T, s=s, edgecolors=edgecolors, **styles)
 
     def draw_rect(self, points, **styles):
         raise NotImplementedError("too annoying")
@@ -2190,17 +2214,136 @@ class X3DAxes(GraphicsAxes3D):
         else:
             self.opts['viewpoint'] = new_opts
 
-    def draw_line(self, points, **styles):
-        line_set = x3d.X3DLine(points, **styles)
+    def draw_line(self, points, indices=None, s=None, riffle=True, line_thickness=None,
+                  edgecolors=None, color=None, glow=None, **styles):
+        if color is None: color = edgecolors
+        if color is None: color = 'black'
+        if line_thickness is None and s is not None:
+            if not nput.is_numeric(s): s = s[0]
+            line_thickness = s / 1000
+        points = np.asanyarray(points)
+        if riffle:
+            if indices is not None:
+                indices = np.asanyarray(indices)
+                if indices.ndim > 1 and indices.shape[-1] > 2:
+                    riff_start = np.arange(indices.shape[-1])
+                    riff_end = np.roll(riff_start, -1)
+                    indices = np.concatenate([
+                        indices[..., riff_start, np.newaxis],
+                        indices[..., riff_end, np.newaxis]
+                    ], axis=-1).reshape(-1, 2)
+                    indices = np.concatenate([indices, np.full((indices.shape[0], 1), -1)], axis=-1)
+            elif points.ndim > 2 and points.shape[-2] > 2:
+                riff_start = np.arange(points.shape[-2])
+                riff_end = np.roll(riff_start, 1)
+                points = np.concatenate([
+                    points[..., riff_start, np.newaxis, :],
+                    points[..., riff_end, np.newaxis, :]
+                ], axis=-2).reshape((-1, 2, 3))
+        if glow is None:
+            glow = color
+
+        if indices is not None:
+            line_set = x3d.X3DIndexedLineSet(points, indices, line_thickness=line_thickness, glow=glow, **styles)
+        else:
+            line_set = x3d.X3DLine(points, line_thickness=line_thickness, glow=glow, **styles)
         self.children.append(line_set)
 
         return line_set
 
-    def draw_disk(self, points, **styles):
-        disk_set = x3d.X3DDisk(points, **styles)
-        self.children.append(disk_set)
+    def draw_disk(self,
+                  points,
+                  radius=None,
+                  color=None,
+                  line_color=None,
+                  edgecolors=None,
+                  s=None,
+                  normal=None,
+                  line_thickness=None,
+                  innerRadius=None,
+                  outerRadius=None,
+                  uv_axes=None,
+                  uv_sign=None,
+                  angle=None,
+                  rotation=None,
+                  solid=None,
+                  **styles):
+        if radius is None and s is not None:
+            radius = s / 100
+        if line_color is None:
+            line_color = edgecolors
 
-        return disk_set
+        if uv_axes is not None:
+            u, v = uv_axes
+            base_ang, base_norm = nput.vec_angles(u, v, return_crosses=True)
+            base_norm = nput.vec_normalize(base_norm)
+            if normal is None:
+                normal = base_norm
+            angs, crosses = nput.vec_angles([0, 0, 1], normal, return_crosses=True, return_norms=False)
+            embedding_axes = nput.rotation_matrix(crosses, angs).T
+            local_x, local_y, local_z = embedding_axes
+            emb_angle, ax2 = nput.vec_angles(local_x, v)
+            if uv_sign is None:
+                # print(np.dot(local_x, v))
+                # print(np.dot(local_y, v))
+                # print(np.dot(local_x, u))
+                # print(np.dot(local_y, u))
+                uv_sign = np.sign(np.dot(local_y, v))
+            emb_angle = uv_sign * emb_angle
+            if rotation is None:
+                rotation = [0, 0, 1, emb_angle]
+            if angle is None:
+                angle = base_ang
+
+        objects = []
+        if line_color is not None:
+            if line_thickness is None:
+                disk_set = x3d.X3DCircle2D(points,
+                                           normal=normal,
+                                           radius=radius,
+                                           glow=line_color,
+                                           rotation=rotation,
+                                           solid=False if solid is None else solid,
+                                           angle=angle,
+                                           **styles
+                                           )
+            else:
+                if color is None:
+                    if innerRadius is None:
+                        innerRadius = line_thickness
+                    if outerRadius is None:
+                        outerRadius = radius
+                disk_set = x3d.X3DTorus(points,
+                                        normal=normal,
+                                        inner_radius=innerRadius if color is None else line_thickness,
+                                        radius=outerRadius if color is None else radius,
+                                        color=line_color,
+                                        solid=solid,
+                                        rotation=rotation,
+                                        angle=angle,
+                                        **styles
+                                        )
+            objects.append(disk_set)
+
+        if color is None and line_color is None:
+            color = 'black'
+        if color is not None:
+            if outerRadius is None:
+                outerRadius = radius
+            disk_set = x3d.X3DDisk2D(points,
+                                     normal=normal,
+                                     inner_radius=innerRadius,
+                                     radius=outerRadius,
+                                     color=color,
+                                     rotation=rotation,
+                                     solid=False if solid is None else solid,
+                                     angle=angle,
+                                     **styles
+                                     )
+            objects.append(disk_set)
+
+        self.children.extend(objects)
+        return objects
 
     def draw_arrow(self, points, **styles):
         points = np.asanyarray(points)
@@ -2219,7 +2362,24 @@ class X3DAxes(GraphicsAxes3D):
         self.children.append(rects)
 
         return rects
+    def draw_point(self, points, color=None, glow=None, **styles):
+        points = np.asanyarray(points)
+        if glow is None: glow = color
+        rects = x3d.X3DPointSet(points, glow=glow, **styles)
+        self.children.append(rects)
 
+        return rects
+    def draw_triangle(self, points, indices=None, **styles):
+        if indices is None:
+            points = np.asanyarray(points)
+            rects = x3d.X3DTriangleSet(points, **styles)
+        else:
+            points = np.asanyarray(points)
+            indices = np.asanyarray(indices)
+            rects = x3d.X3DIndexedTriangleSet(points, indices, **styles)
+        self.children.append(rects)
+
+        return rects
     def draw_poly(self, points, **styles):
         raise NotImplementedError("2D")
 
@@ -2229,8 +2389,11 @@ class X3DAxes(GraphicsAxes3D):
 
         return spheres
 
-    def draw_cylinder(self, starts, ends, rads, **styles):
-        cyls = x3d.X3DCylinder(starts, ends, radius=rads, **styles)
+    def draw_cylinder(self, starts, ends, rads, capped=False, **styles):
+        if capped:
+            cyls = x3d.X3DCappedCylinder(starts, ends, radius=rads, **styles)
+        else:
+            cyls = x3d.X3DCylinder(starts, ends, radius=rads, **styles)
         self.children.append(cyls)
 
         return cyls
